/*-----------------------------------------------* 
Project  : Government Analytics in LAC
File: 	   Report on survey responses by country		
Author(s): Flavia Sacco
Date: 	   January 5, 2024						
Input  :   All MIS questionnaires data from SurveyCTO
*-----------------------------------------------*/



//Locals for each MIS queationnaire
local education "EDMIS.dta"
local health "HealthMIS.dta"
local tax "TaxesMIS.dta"
local procurement "ProcurementMIS.dta"
local finance "PFMIS.dta"
local human "HRMIS.dta"

//Open the data
use "$input\`education'", clear

//Additional cleanning
*For Honduras, we have more than 1 questionnaire by MIS, keeping the ones coming from the corresponding Ministry
drop if q1==20 & q5=="UNIVERSIDAD AUTONOMA DE HONDURAS (UNAH)"
drop if q1==20 & q5=="Instituto Hondureño de Seguridad Social (IHSS)"
drop if q1==20 & q5=="Comisión Nacional de Bancos y Seguros (CNBS)"
drop if q1==20 & q5=="Banco Central de Honduras (BCH)"
drop if q3=="Flavia" 
drop if consent_to_answer==1
drop if q1==32 & q3==""
drop if q1==21 //Jamaica hasnt submitted yet


//Replacing 900 and 998 with missing 
replace q9=. if q9==900 | q9==999
replace q11=. if q11==900 | q11==998
replace q14=. if q14==900 | q14==998
replace q15=. if q15==900 | q15==998
replace q19=. if q19==900 | q19==998
replace q20=. if q20==900 | q20==998
replace q21=. if q21==900 | q21==998
replace q22=. if q22==900 | q22==998
replace q24=. if q24==900 | q24==998
replace q25=. if q25==900 | q25==998
replace q26=. if q26==900 | q26==998
replace q28=. if q28==900 | q28==998


//Labelling multiple choice options

*Agents of government that use data analytics products
label var q12_1 "Non-managerial staff"
label var q12_2 "Managerial staff"
label var q12_3 "Politicians"
label var q12_4 "No agent uses data analytics products to inform decisions"
label var q12_700 "Other"
label var q12_900 "Don't know"
label var q12_998 "Prefer not to respond"


*Main uses of data analytics products
label var q13_1 "Monitoring"
label var q13_2 "Accountability"
label var q13_3 "Transparency towards citizens"
label var q13_4 "Policy evaluation"
label var q13_5 "Policy design"
label var q13_6 "Data analytics products are not used"
label var q13_700 "Other"
label var q13_900 "Don't know"
label var q13_998 "Prefer not to respond"



*Type of data analytics 
label var q16_1 "Descriptive analytics"
label var q16_2 "Diagnostic analytics"
label var q16_3 "Predictive analytics"
label var q16_900 "Don't know"
label var q16_998 "Prefer not to respond"

*Main drivers for collaboration
label var q23_1 "Technical assistance"
label var q23_2 "Data access"
label var q23_3 "Innovations in data analytics"
label var q23_4 "Financial support"
label var q23_6 "There is no strategy to collaborate on data analytics with academics, etc."
label var q23_700 "Other"
label var q23_900 "Don't know"
label var q23_998 "Prefer not to respond"



//Ordering data
order q1 fully_digitized elem_have q24 q25 q28 agency_use q26 q16 elem_used q20 q14 q21 q22 q19 q15 q13 use_analytics agents

//Change this for every country
save "$input\EDMIS_clean.dta", replace
